@@ -13,7 +13,7 @@ from opencmiss.utils.zinc.field import findOrCreateFieldCoordinates, findOrCreat
     findOrCreateFieldNodeGroup, findOrCreateFieldStoredMeshLocation, findOrCreateFieldStoredString
 from opencmiss.utils.zinc.finiteelement import getMaximumElementIdentifier, getMaximumNodeIdentifier
 from opencmiss.zinc.element import Element
-from opencmiss.zinc.field import Field
+from opencmiss.zinc.field import Field, FieldGroup
 from opencmiss.zinc.node import Node
 from scaffoldmaker.annotation.annotationgroup import AnnotationGroup, getAnnotationGroupForTerm
 from scaffoldmaker.annotation.heart_terms import get_heart_term
@@ -46,11 +46,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
             'Human 1',
             'Mouse 1',
             'Pig 1',
-            'Rat 1',
-            'Unit Human 1',
-            'Unit Mouse 1',
-            'Unit Pig 1',
-            'Unit Rat 1']
+            'Rat 1']
 
     @staticmethod
     def getDefaultOptions(parameterSetName='Default'):
@@ -60,7 +56,6 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         isMouse = 'Mouse' in parameterSetName
         isPig = 'Pig' in parameterSetName
         isRat = 'Rat' in parameterSetName
-        notUnitScale = 'Unit' not in parameterSetName
         # only works with particular numbers of elements around
         options['Number of elements around LV free wall'] = 7
         options['Number of elements around RV free wall'] = 7
@@ -101,8 +96,8 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         if isHuman:
             options['LV outer height'] = 0.9
         elif isMouse or isRat:
-            options['LV outer height'] = 0.85
-            options['Base height'] = 0.23
+            options['LV outer height'] = 0.9
+            options['Base height'] = 0.18
             options['Base thickness'] = 0.08
             options['Fibrous ring thickness'] = 0.005
             options['LV outlet inner diameter'] = 0.21
@@ -177,8 +172,6 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         '''
         dependentChanges = MeshType_3d_heartventricles1.checkOptions(options)
         # only works with particular numbers of elements around
-        options['Number of elements around RV free wall'] = 7
-        options['Number of elements around atrial septum'] = 3
         # start with limitations from atria1:
         #if options['Number of elements around atrial septum'] < 2:
         #    options['Number of elements around atrial septum'] = 2
@@ -189,6 +182,26 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 options[key] = 6
             elif options[key] > 10:
                 options[key] = 10
+        if options['Collapse RV columns']:
+            if options['Number of elements around right atrium free wall'] != 8:
+                options['Number of elements around right atrium free wall'] = 8
+                dependentChanges = True
+            if options['Number of elements around atrial septum'] != 3:
+                options['Number of elements around atrial septum'] = 3
+                dependentChanges = True
+            if options['Number of elements around RV free wall'] != 9:
+                options['Number of elements around RV free wall'] = 9
+                dependentChanges = True
+        else:
+            if options['Number of elements around right atrium free wall'] not in [6, 8]:
+                options['Number of elements around right atrium free wall'] = 8
+                dependentChanges = True
+            if options['Number of elements around atrial septum'] != 3:
+                options['Number of elements around atrial septum'] = 3
+                dependentChanges = True
+            if options['Number of elements around RV free wall'] != 7:
+                options['Number of elements around RV free wall'] = 7
+                dependentChanges = True
         # Supports only 6 or 8 elements around right atrium free wall:
         if options['Number of elements around right atrium free wall'] <= 6:
             options['Number of elements around right atrium free wall'] = 6
@@ -247,12 +260,14 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         """
         elementsCountAroundLVFreeWall = options['Number of elements around LV free wall']
         elementsCountAroundRVFreeWall = options['Number of elements around RV free wall']
-        elementsCountAroundLV = elementsCountAroundLVFreeWall + elementsCountAroundRVFreeWall
-        elementsCountAroundVSeptum = elementsCountAroundRVFreeWall
-        elementsCountAroundRV = 2*elementsCountAroundRVFreeWall
         elementsCountUpLVApex = options['Number of elements up LV apex']
         elementsCountUpRV = options['Number of elements up RV']
         elementsCountUpLV = elementsCountUpLVApex + elementsCountUpRV
+        collapseRVColumns = options['Collapse RV columns']
+        elementsCountAroundVSeptum = (elementsCountAroundRVFreeWall - 2) if collapseRVColumns \
+            else elementsCountAroundRVFreeWall
+        elementsCountAroundLV = elementsCountAroundLVFreeWall + elementsCountAroundVSeptum
+        elementsCountAroundRV = elementsCountAroundRVFreeWall + elementsCountAroundVSeptum
         elementsCountAroundAtrialSeptum = options['Number of elements around atrial septum']
         elementsCountAroundLeftAtriumFreeWall = options['Number of elements around left atrium free wall']
         elementsCountAroundLeftAtrium = elementsCountAroundLeftAtriumFreeWall + elementsCountAroundAtrialSeptum
@@ -296,8 +311,6 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         coordinates = findOrCreateFieldCoordinates(fm)
         cache = fm.createFieldcache()
 
-        mesh = fm.findMeshByDimension(3)
-
         # generate heartventricles1 model to add base plane to
         annotationGroups = MeshType_3d_heartventricles1.generateBaseMesh(region, options)
 
@@ -307,26 +320,24 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         rvGroup = getAnnotationGroupForTerm(annotationGroups, get_heart_term("right ventricle myocardium"))
         vSeptumGroup = getAnnotationGroupForTerm(annotationGroups, get_heart_term("interventricular septum"))
         conusArteriosusGroup = AnnotationGroup(region, get_heart_term("conus arteriosus"))
-        annotationGroups += [ conusArteriosusGroup ]
         # av boundary nodes are put in left and right fibrous ring groups only so they can be found by heart1
         lFibrousRingGroup = AnnotationGroup(region, get_heart_term("left fibrous ring"))
         rFibrousRingGroup = AnnotationGroup(region, get_heart_term("right fibrous ring"))
+        # temporary groups for making face annotations
+        lvOutletGroup = AnnotationGroup(region, ("LV outlet", "None"))
+        mitralAorticCurtainGroup = AnnotationGroup(region, ("Mitral aortic curtain", "None"))
+        supraventricularCrestGroup = AnnotationGroup(region, ("Supraventricular crest", "None"))
+        annotationGroups += [conusArteriosusGroup, lFibrousRingGroup, rFibrousRingGroup, lvOutletGroup,
+                             mitralAorticCurtainGroup, supraventricularCrestGroup]
 
         # annotation fiducial points
         markerGroup = findOrCreateFieldGroup(fm, "marker")
-        markerName = findOrCreateFieldStoredString(fm, name="marker_name")
-        markerLocation = findOrCreateFieldStoredMeshLocation(fm, mesh, name="marker_location")
-
-        nodes = fm.findNodesetByFieldDomainType(Field.DOMAIN_TYPE_NODES)
-        markerPoints = findOrCreateFieldNodeGroup(markerGroup, nodes).getNodesetGroup()
-        markerTemplateInternal = nodes.createNodetemplate()
-        markerTemplateInternal.defineField(markerName)
-        markerTemplateInternal.defineField(markerLocation)
 
         #################
         # Create nodes
         #################
 
+        nodes = fm.findNodesetByFieldDomainType(Field.DOMAIN_TYPE_NODES)
         nodetemplate = nodes.createNodetemplate()
         nodetemplate.defineField(coordinates)
         nodetemplate.setValueNumberOfVersions(coordinates, -1, Node.VALUE_LABEL_VALUE, 1)
@@ -351,15 +362,21 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         newCoordinates = fm.createFieldAdd(fm.createFieldMatrixMultiply(3, rotationMatrix, coordinates), ventriclesOffset)
         fieldassignment = coordinates.createFieldassignment(newCoordinates)
         fieldassignment.setNodeset(nodes)
+        # marker points are not rotated
+        fieldassignment.setConditionalField(fm.createFieldNot(markerGroup.getFieldNodeGroup(nodes)))
         fieldassignment.assign()
 
         # discover ventricles top LV inner, RV inner, V Outer nodes, coordinates and derivatives
         startLVInnerNodeId = 2 + (elementsCountUpLV - 1)*elementsCountAroundLV
         lvInnerNodeId = [ (startLVInnerNodeId + n1) for n1 in range(elementsCountAroundLV) ]
-        startRVInnerNodeId = startLVInnerNodeId + elementsCountAroundLV + elementsCountAroundRVFreeWall + 1 + elementsCountAroundRV*(elementsCountUpRV - 1)
+        startRVInnerNodeId = startLVInnerNodeId + elementsCountAroundLV + elementsCountAroundVSeptum + 1 \
+                             + elementsCountAroundRV*(elementsCountUpRV - 1)
         rvInnerNodeId = [ (startRVInnerNodeId + n1) for n1 in range(elementsCountAroundRV) ]
-        startVOuterNodeId = startRVInnerNodeId + elementsCountAroundRV + 1 + (elementsCountUpLV - 1)*elementsCountAroundLV
-        vOuterNodeId = [ (startVOuterNodeId + n1) for n1 in range(elementsCountAroundLV) ]
+        startVOuterNodeId = startRVInnerNodeId + elementsCountAroundRV + 1 \
+                            + elementsCountUpLVApex*elementsCountAroundLV \
+                            + (elementsCountUpLV - elementsCountUpLVApex - 1) * \
+                            (elementsCountAroundLVFreeWall + elementsCountAroundRVFreeWall)
+        vOuterNodeId = [ (startVOuterNodeId + n1) for n1 in range(elementsCountAroundLVFreeWall + elementsCountAroundRVFreeWall) ]
         for nodeId in [ lvInnerNodeId, rvInnerNodeId, vOuterNodeId ]:
             vx  = []
             vd1 = [] if (nodeId is rvInnerNodeId) else None
@@ -476,8 +493,12 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 for c in range(3):
                     lavd2[0][0][noa][c] += lavd1[0][0][noa][c]
                     lavd2[1][0][noa][c] += lavd1[1][0][noa][c]
-        elementsCountRVHanging = 2 if (elementsCountAroundRightAtriumFreeWall == 8) else 0
-        elementsCountRVFreeWallRegular = 3 + elementsCountRVHanging
+        if collapseRVColumns:
+            elementsCountRVHanging = 0
+            elementsCountRVFreeWallRegular = 5
+        else:
+            elementsCountRVHanging = 2 if (elementsCountAroundRightAtriumFreeWall == 8) else 0
+            elementsCountRVFreeWallRegular = 3 + elementsCountRVHanging
         for n1 in range(elementsCountRVFreeWallRegular + 1):
             noa = n1
             niv = n1
@@ -535,7 +556,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
 
         # copy derivative 3 from av points to LV outlet at centre, left and right cfb; negate as d1 is reversed:
         lvOutletOuterd3[0] = [ -d for d in lavd3[1][0][0] ]
-        lvOutletOuterd3[1] = [ -d for d in ravd3[1][0][-2] ]
+        lvOutletOuterd3[1] = [ -d for d in ravd3[1][0][elementsCountAroundRightAtriumFreeWall - 1] ]
         lvOutletOuterd3[-1] = [ -d for d in lavd3[1][0][1] ]
 
         # create point above anterior ventricular septum end
@@ -553,52 +574,57 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         avsd3 = interp.interpolateHermiteLagrangeDerivative(lvInnerx[0], lvInnerd2[0], avsx, 1.0)
         lavd2[1][0][2] = pd1[2]
 
-        # create points on bottom and top of RV supraventricular crest
-        ns = elementsCountAroundRVFreeWall//2 + 1
-        nf = elementsCountAroundRVFreeWall + 2
-        xis = 0.667
-        xif = 1.0 - xis
-        mx = [ xis*rvInnerx[ns][0] + xif*rvInnerx[nf][0], xis*rvInnerx[ns][1] + xif*rvInnerx[nf][1], -(fibrousRingThickness + baseThickness) ]
-        md2 = [ (rvInnerx[nf][c] - rvInnerx[ns][c]) for c in range(3) ]
-        sd1 = [ -d for d in ravd2[0][0][ravsvcn2] ]
+        # create points on top and bottom of RV supraventricular crest
+
+        ns = elementsCountAroundLVFreeWall + elementsCountAroundRVFreeWall - 3
+        nf = 2
+        sx = vOuterx[ns]
+        sd2 = vOuterd2[ns]
+        fx = lvOutletOuterx[nf]
+        fd2 = vector.setMagnitude([lvOutletInnerx[nf][c] - lvOutletOuterx[nf][c] for c in range(3)],
+                                  vector.magnitude(sd2))
+        scale = interp.computeCubicHermiteDerivativeScaling(sx, sd2, fx, fd2)
+        px, pd2 = interp.sampleCubicHermiteCurvesSmooth([sx, fx], [[d*scale for d in sd2], [d*scale for d in fd2]], 2,
+            derivativeMagnitudeStart=vector.magnitude(sd2))[0:2]
+        svcox = px[1]
+        sd1 = [-d for d in ravd2[1][0][ravsvcn2]]
+        fd1 = [rvOutletOuterd1[2][c] + rvOutletd2[c] for c in range(3)]
+        pd1 = interp.smoothCubicHermiteDerivativesLine(
+            [ravx[1][0][ravsvcn2], svcox, rvOutletOuterx[1]], [sd1, zero, fd1],
+            fixStartDerivative=True, fixEndDerivative=True)
+        svcod1 = pd1[1]
+        svcod2 = pd2[1]
+        svcod3 = vector.setMagnitude(vector.crossproduct3(svcod1, svcod2), baseThickness)
+        lvOutletOuterd3[nf] = [-d for d in pd2[2]]
+        # set a reasonable value for next d2 up on right fibrous ring by supraventricular crest 1
+        sd12 = [(svcod2[c] - svcod1[c]) for c in range(3)]
+        pd2 = interp.smoothCubicHermiteDerivativesLine([svcox, ravx[1][0][ravsvcn1]], [sd12, ravd2[1][0][ravsvcn1]],
+            fixStartDerivative=True, fixEndDirection=True)
+        ravd2[1][0][ravsvcn1] = pd2[1]
+
+        ns = elementsCountAroundRVFreeWall - 3
+        nf = elementsCountAroundRVFreeWall + 2  # finish on septum
+        svcix = [svcox[c] - svcod3[c] for c in range(3)]
+        sd1 = [-d for d in ravd2[0][0][ravsvcn2]]
         if elementsCountRVHanging == 0:
             for c in range(3):
                 sd1[c] += ravd1[0][0][ravsvcn2][c]
-        fd1 = [ (rvOutletInnerd1[2][c] + rvOutletd2[c]) for c in range(3) ]
-        pd1 = interp.smoothCubicHermiteDerivativesLine([ ravx[0][0][ravsvcn2], mx, rvOutletInnerx[1] ], [ sd1, zero, fd1 ],
-            fixStartDerivative=True, fixEndDerivative = True)
-        pd2 = interp.smoothCubicHermiteDerivativesLine([ rvInnerx[ns], mx, rvInnerx[nf] ], [ rvInnerd2[ns], md2, [ -d for d in rvInnerd2[nf] ] ],
-            fixStartDerivative = True, fixEndDerivative = True)
-        svcix = [ mx[0], mx[1], mx[2] ]  # list components to avoid reference bug
+        fd1 = [rvOutletOuterd1[2][c] + rvOutletd2[c] for c in range(3)]
+        pd1 = interp.smoothCubicHermiteDerivativesLine(
+            [ravx[0][0][ravsvcn2], svcix, rvOutletOuterx[1]], [sd1, zero, fd1],
+            fixStartDerivative=True, fixEndDerivative=True)
+        pd2 = interp.smoothCubicHermiteDerivativesLine(
+            [rvInnerx[ns], svcix, rvInnerx[nf]], [rvInnerd2[ns], svcod2, [-d for d in rvInnerd2[nf]]],
+            fixStartDerivative=True, fixEndDerivative=True, fixAllDirections=True)
         svcid1 = pd1[1]
         svcid2 = pd2[1]
-        svcid3 = vector.setMagnitude(vector.crossproduct3(svcid1, svcid2), baseThickness)
-        sd2 = [ (svcid2[c] - svcid1[c]) for c in range(3) ]
-        pd2 = interp.smoothCubicHermiteDerivativesLine([ mx, ravx[0][0][ravsvcn1] ], [ sd2, ravd2[0][0][ravsvcn1] ],
-            fixStartDerivative=True, fixEndDirection = True)
+        svcid3 = svcod3
+        # set a reasonable value for next d2 up on right fibrous ring by supraventricular crest 1
+        sd12 = [ (svcid2[c] - svcid1[c]) for c in range(3) ]
+        pd2 = interp.smoothCubicHermiteDerivativesLine([svcix, ravx[0][0][ravsvcn1]], [sd12, ravd2[0][0][ravsvcn1]],
+            fixStartDerivative=True, fixEndDirection=True)
         ravd2[0][0][ravsvcn1] = pd2[1]
 
-        mx = [ (mx[c] + svcid3[c]) for c in range(3) ]
-        md2 = svcid2
-        nf = 2
-        sd1 = [ -d for d in ravd2[1][0][ravsvcn2] ]
-        if elementsCountRVHanging == 0:
-            for c in range(3):
-                sd1[c] += ravd1[1][0][ravsvcn2][c]
-        fd1 = [ (rvOutletOuterd1[2][c] + rvOutletd2[c]) for c in range(3) ]
-        pd1 = interp.smoothCubicHermiteDerivativesLine([ ravx[1][0][ravsvcn2], mx, rvOutletOuterx[1] ], [ sd1, zero, fd1 ],
-            fixStartDerivative=True, fixEndDerivative = True)
-        pd2 = interp.smoothCubicHermiteDerivativesLine([ mx, lvOutletOuterx[nf] ], [ md2, svcid2 ], fixStartDirection=True)  # , instrument=True)
-        svcox = copy.copy(mx)
-        svcod1 = pd1[1]
-        svcod2 = pd2[0]
-        svcod3 = svcid3
-        lvOutletOuterd3[nf] = [ -d for d in pd2[1] ]
-        sd2 = [ (svcod2[c] - svcod1[c]) for c in range(3) ]
-        pd2 = interp.smoothCubicHermiteDerivativesLine([ mx, ravx[1][0][ravsvcn1] ], [ sd2, ravd2[1][0][ravsvcn1] ],
-            fixStartDerivative=True, fixEndDirection = True)
-        ravd2[1][0][ravsvcn1] = pd2[1]
-
         # LV outlet nodes
         lvOutletNodeId = [ [], [] ]
         for n3 in range(2):
@@ -729,8 +755,12 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         # Create elements
         #################
 
+        mesh = fm.findMeshByDimension(3)
         heartMeshGroup = heartGroup.getMeshGroup(mesh)
         lvMeshGroup = lvGroup.getMeshGroup(mesh)
+        lvOutletMeshGroup = lvOutletGroup.getMeshGroup(mesh)
+        mitralAorticCurtainMeshGroup = mitralAorticCurtainGroup.getMeshGroup(mesh)
+        supraventricularCrestMeshGroup = supraventricularCrestGroup.getMeshGroup(mesh)
         rvMeshGroup = rvGroup.getMeshGroup(mesh)
         vSeptumMeshGroup = vSeptumGroup.getMeshGroup(mesh)
         conusArteriosusMeshGroup = conusArteriosusGroup.getMeshGroup(mesh)
@@ -815,7 +845,6 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
             nids = None
             scalefactors = None
             meshGroups = [ heartMeshGroup, rvMeshGroup ]
-            addMarker = None
 
             noa = e
             niv = e
@@ -826,7 +855,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                     niv -= 1
             nivp = niv + 1
             nov = elementsCountAroundLVFreeWall + niv
-            novp = (nov + 1)%elementsCountAroundLV
+            novp = (nov + 1) % (elementsCountAroundLVFreeWall + elementsCountAroundRVFreeWall)
             if e == -1:
                 # crux / posterior interventricular sulcus, collapsed to 6 node wedge
                 nids = [ lvInnerNodeId[elementsCountAroundLVFreeWall], rvInnerNodeId[nivp], lavNodeId[0][0][elementsCountAroundLeftAtriumFreeWall], ravNodeId[0][0][noa + 1],
@@ -935,7 +964,6 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 # outer infundibulum 5, above septum
                 nids = [ rvInnerNodeId[niv - 1],   rvInnerNodeId[niv], rvOutletNodeId[0][5], rvOutletNodeId[0][0],
                                       avsNodeId, lvOutletNodeId[1][3], rvOutletNodeId[1][5], rvOutletNodeId[1][0] ]
-                addMarker = { "name" : "Pulmonary valve-RV", "xi" : [ 1.0, 1.0, 0.0 ] }
                 eft1 = tricubichermite.createEftNoCrossDerivatives()
                 setEftScaleFactorIds(eft1, [1], [])
                 scalefactors = [ -1.0 ]
@@ -959,13 +987,6 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
             #print('create element rv base r1', elementIdentifier, result, result2, result3, nids)
             elementIdentifier += 1
 
-            if addMarker:
-                markerPoint = markerPoints.createNode(nodeIdentifier, markerTemplateInternal)
-                nodeIdentifier += 1
-                cache.setNode(markerPoint)
-                markerName.assignString(cache, addMarker["name"])
-                markerLocation.assignMeshLocation(cache, element, addMarker["xi"])
-
             for meshGroup in meshGroups:
                 meshGroup.addElement(element)
 
@@ -1077,8 +1098,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
             eft1 = eft
             nids = None
             scalefactors = None
-            meshGroups = [ heartMeshGroup, lvMeshGroup ]
-            addMarker = None
+            meshGroups = [ heartMeshGroup, lvMeshGroup, lvOutletMeshGroup ]
 
             eft1 = tricubichermite.createEftNoCrossDerivatives()
             setEftScaleFactorIds(eft1, [1], [])
@@ -1110,7 +1130,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 ln_map = [ 1, 2, 3, 3, 4, 5, 6, 6 ]
                 remapEftLocalNodes(eft1, 6, ln_map)
             elif e == 2:
-                # 7 node collapsed element where lv outlet ring expands into
+                # 7 node collapsed element where lv outlet ring expands into LV freewall
                 nids = [ lavNodeId[0][0][2], lavNodeId[0][0][1], lvOutletNodeId[0][4], lvOutletNodeId[0][5],
                          lavNodeId[1][0][2], lavNodeId[1][0][1], lvOutletNodeId[1][4] ]
                 remapEftNodeValueLabel(eft1, [ 1, 2, 5 ], Node.VALUE_LABEL_D_DS1, [ ( Node.VALUE_LABEL_D_DS1, [1] ) ])
@@ -1124,6 +1144,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 remapEftNodeValueLabel(eft1, [ 6 ], Node.VALUE_LABEL_D_DS3, [ ( Node.VALUE_LABEL_D_DS3, [1] ) ])
                 ln_map = [ 1, 2, 3, 4, 5, 6, 7, 6 ]
                 remapEftLocalNodes(eft1, 7, ln_map)
+                meshGroups.append(mitralAorticCurtainMeshGroup)
             elif e == 3:
                 # 6 node wedge element bridge/curtain between mitral and aortic valve orifices
                 no = e - 4
@@ -1140,6 +1161,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 remapEftNodeValueLabel(eft1, [ 5, 6, 7, 8 ], Node.VALUE_LABEL_D_DS2, [])
                 ln_map = [ 1, 2, 3, 4, 5, 6, 5, 6 ]
                 remapEftLocalNodes(eft1, 6, ln_map)
+                meshGroups.append(mitralAorticCurtainMeshGroup)
             elif e == 4:
                 # tetrahedral cfb shim-bridge connector element
                 ni = elementsCountAroundLVFreeWall + elementsCountAroundAtrialSeptum
@@ -1162,8 +1184,6 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 no = e - 5
                 ni = elementsCountAroundLVFreeWall + elementsCountAroundAtrialSeptum + no
                 nids = [ lvInnerNodeId[ni], lvInnerNodeId[ni + 1], lvOutletNodeId[0][no], lvOutletNodeId[0][no + 1], lvOutletNodeId[1][no], lvOutletNodeId[1][no + 1] ]
-                if nids[2] == lvOutletNodeId[0][2]:
-                    addMarker = { "name" : "Aortic Valve-Coronary vessel", "xi" : [ 0.0, 1.0, 0.0 ] }
                 if no == 0:
                     remapEftNodeValueLabel(eft1, [ 1 ], Node.VALUE_LABEL_D_DS2, [ ( Node.VALUE_LABEL_D_DS1, [] ), ( Node.VALUE_LABEL_D_DS2, [] ) ])
                     remapEftNodeValueLabel(eft1, [ 1 ], Node.VALUE_LABEL_D_DS3, [ ( Node.VALUE_LABEL_D_DS2, [] ), ( Node.VALUE_LABEL_D_DS3, [] ) ])
@@ -1184,13 +1204,6 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
             #print('create element lv base r2', elementIdentifier, result, result2, result3, nids)
             elementIdentifier += 1
 
-            if addMarker:
-                markerPoint = markerPoints.createNode(nodeIdentifier, markerTemplateInternal)
-                nodeIdentifier += 1
-                cache.setNode(markerPoint)
-                markerName.assignString(cache, addMarker["name"])
-                markerLocation.assignMeshLocation(cache, element, addMarker["xi"])
-
             for meshGroup in meshGroups:
                 meshGroup.addElement(element)
 
@@ -1245,6 +1258,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 remapEftNodeValueLabel(eft1, [ 8 ], Node.VALUE_LABEL_D_DS3, [ ( Node.VALUE_LABEL_D_DS2, [] ) ])
                 ln_map = [ 1, 2, 3, 4, 5, 6, 7, 7 ]
                 remapEftLocalNodes(eft1, 7, ln_map)
+                meshGroups.append(supraventricularCrestMeshGroup)
             elif e == 2:
                 # 8-node rv crest row 2 element 1
                 rvin1 = -elementsCountAroundAtrialSeptum - 2
@@ -1265,6 +1279,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 remapEftNodeValueLabel(eft1, [ 8 ], Node.VALUE_LABEL_D_DS1, [ ( Node.VALUE_LABEL_D_DS1, [] ), ( Node.VALUE_LABEL_D_DS3, [1] ) ])
                 remapEftNodeValueLabel(eft1, [ 8 ], Node.VALUE_LABEL_D_DS2, [ ( Node.VALUE_LABEL_D_DS3, [1] ) ])
                 remapEftNodeValueLabel(eft1, [ 8 ], Node.VALUE_LABEL_D_DS3, [ ( Node.VALUE_LABEL_D_DS2, [] ) ])
+                meshGroups.append(supraventricularCrestMeshGroup)
             elif e == 3:
                 # 8-node wedge rv crest row 2 element 2
                 rvin1 = -elementsCountAroundAtrialSeptum - 2
@@ -1283,7 +1298,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 remapEftNodeValueLabel(eft1, [ 7 ], Node.VALUE_LABEL_D_DS1, [ ( Node.VALUE_LABEL_D_DS1, [] ), ( Node.VALUE_LABEL_D_DS3, [] ) ])
                 remapEftNodeValueLabel(eft1, [ 7 ], Node.VALUE_LABEL_D_DS2, [ ( Node.VALUE_LABEL_D_DS3, [1] ) ])
                 remapEftNodeValueLabel(eft1, [ 7 ], Node.VALUE_LABEL_D_DS3, [ ( Node.VALUE_LABEL_D_DS2, [] ) ])
-                meshGroups += [ conusArteriosusMeshGroup ]
+                meshGroups += [conusArteriosusMeshGroup, supraventricularCrestMeshGroup]
             elif e == 4:
                 # 8-node rv crest inner 4 by rv outlet
                 rvin1 = -elementsCountAroundAtrialSeptum - 2
@@ -1312,7 +1327,7 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
                 remapEftNodeValueLabel(eft1, [ 7 ], Node.VALUE_LABEL_D2_DS1DS2, [ ( Node.VALUE_LABEL_D_DS2, [] ) ])
                 remapEftNodeValueLabel(eft1, [ 8 ], Node.VALUE_LABEL_D_DS2, [ ( Node.VALUE_LABEL_D_DS1, [1] ) ])
                 remapEftNodeValueLabel(eft1, [ 8 ], Node.VALUE_LABEL_D_DS1, [ ( Node.VALUE_LABEL_D_DS2, [] ) ])
-                meshGroups += [ conusArteriosusMeshGroup ]
+                meshGroups += [conusArteriosusMeshGroup, supraventricularCrestMeshGroup]
             else:
                 continue
 
@@ -1338,12 +1353,14 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         assert isinstance(meshrefinement, MeshRefinement)
         elementsCountAroundLVFreeWall = options['Number of elements around LV free wall']
         elementsCountAroundRVFreeWall = options['Number of elements around RV free wall']
-        elementsCountAroundLV = elementsCountAroundLVFreeWall + elementsCountAroundRVFreeWall
-        elementsCountAroundVSeptum = elementsCountAroundRVFreeWall
-        elementsCountAroundRV = 2*elementsCountAroundRVFreeWall
         elementsCountUpLVApex = options['Number of elements up LV apex']
         elementsCountUpRV = options['Number of elements up RV']
         elementsCountUpLV = elementsCountUpLVApex + elementsCountUpRV
+        collapseRVColumns = options['Collapse RV columns']
+        elementsCountAroundVSeptum = (elementsCountAroundRVFreeWall - 2) if collapseRVColumns \
+            else elementsCountAroundRVFreeWall
+        elementsCountAroundLV = elementsCountAroundLVFreeWall + elementsCountAroundVSeptum
+        elementsCountAroundRV = elementsCountAroundRVFreeWall + elementsCountAroundVSeptum
         elementsCountAroundAtrialSeptum = options['Number of elements around atrial septum']
         elementsCountAroundLeftAtriumFreeWall = options['Number of elements around left atrium free wall']
         elementsCountAroundRightAtriumFreeWall = options['Number of elements around right atrium free wall']
@@ -1405,3 +1422,44 @@ class MeshType_3d_heartventriclesbase1(Scaffold_base):
         New face annotation groups are appended to this list.
         """
         MeshType_3d_heartventricles1.defineFaceAnnotations(region, options, annotationGroups)
+        conusArteriosusGroup = getAnnotationGroupForTerm(annotationGroups, get_heart_term("conus arteriosus"))
+        lFibrousRingGroup = getAnnotationGroupForTerm(annotationGroups, get_heart_term("left fibrous ring"))
+        rFibrousRingGroup = getAnnotationGroupForTerm(annotationGroups, get_heart_term("right fibrous ring"))
+        # temporary groups
+        lvOutletGroup = getAnnotationGroupForTerm(annotationGroups, ("LV outlet", "None"))
+        mitralAorticCurtainGroup = getAnnotationGroupForTerm(annotationGroups, ("Mitral aortic curtain", "None"))
+        supraventricularCrestGroup = getAnnotationGroupForTerm(annotationGroups, ("Supraventricular crest", "None"))
+
+        fm = region.getFieldmodule()
+        if (lFibrousRingGroup.getDimension() <= 0) or (rFibrousRingGroup.getDimension() <= 0):
+            mesh2d = fm.findMeshByDimension(2)
+            is_exterior = fm.createFieldIsExterior()
+            is_face_xi1_0 = fm.createFieldIsOnFace(Element.FACE_TYPE_XI1_0)
+            is_face_xi2_0 = fm.createFieldIsOnFace(Element.FACE_TYPE_XI2_0)
+            is_face_xi2_1 = fm.createFieldIsOnFace(Element.FACE_TYPE_XI2_1)
+            lvGroup = getAnnotationGroupForTerm(annotationGroups, get_heart_term("left ventricle myocardium"))
+            rvGroup = getAnnotationGroupForTerm(annotationGroups, get_heart_term("right ventricle myocardium"))
+            is_lv = lvGroup.getFieldElementGroup(mesh2d)
+            is_rv = rvGroup.getFieldElementGroup(mesh2d)
+            is_lvo = lvOutletGroup.getFieldElementGroup(mesh2d)
+            is_mac = mitralAorticCurtainGroup.getFieldElementGroup(mesh2d)
+            is_lfr = fm.createFieldAnd(
+                is_exterior,
+                fm.createFieldOr(
+                    fm.createFieldAnd(fm.createFieldAnd(is_lv, fm.createFieldNot(is_lvo)), is_face_xi2_1),
+                    fm.createFieldAnd(is_mac, is_face_xi2_0)))
+            lFibrousRingGroup.getMeshGroup(mesh2d).addElementsConditional(is_lfr)
+            is_ca = conusArteriosusGroup.getFieldElementGroup(mesh2d)
+            is_svc = supraventricularCrestGroup.getFieldElementGroup(mesh2d)
+            is_rfr = fm.createFieldAnd(
+                is_exterior,
+                fm.createFieldAnd(
+                    is_rv,
+                    fm.createFieldOr(
+                        fm.createFieldAnd(is_face_xi2_1, fm.createFieldNot(is_ca)),
+                        fm.createFieldAnd(is_face_xi1_0, is_svc))))
+            rFibrousRingGroup.getMeshGroup(mesh2d).addElementsConditional(is_rfr)
+
+        annotationGroups.remove(lvOutletGroup)
+        annotationGroups.remove(mitralAorticCurtainGroup)
+        annotationGroups.remove(supraventricularCrestGroup)
