@@ -5,10 +5,10 @@ Utility functions for easing use of Zinc API.
 from opencmiss.utils.zinc.field import findOrCreateFieldCoordinates
 from opencmiss.utils.zinc.general import ChangeManager
 from opencmiss.zinc.context import Context
-from opencmiss.zinc.element import MeshGroup
+from opencmiss.zinc.element import Mesh, MeshGroup
 from opencmiss.zinc.field import Field, FieldGroup
 from opencmiss.zinc.fieldmodule import Fieldmodule
-from opencmiss.zinc.node import Node
+from opencmiss.zinc.node import Node, Nodeset
 from opencmiss.zinc.result import RESULT_OK
 from scaffoldmaker.utils import interpolation as interp
 from scaffoldmaker.utils import vector
@@ -146,6 +146,31 @@ def createFaceMeshGroupExteriorOnFace(fieldmodule : Fieldmodule, elementFaceType
         del isOnFace
     return faceMeshGroup
 
+
+def get_highest_dimension_mesh(fieldmodule : Fieldmodule) -> Mesh:
+    '''
+    Get highest dimension non-empty mesh.
+    :return: Zinc Mesh or None if all are empty.
+    '''
+    for dimension in range(3, 0, -1):
+        mesh = fieldmodule.findMeshByDimension(dimension)
+        if mesh.getSize() > 0:
+            return mesh
+    return None
+
+
+def get_next_unused_node_identifier(nodeset: Nodeset, start_identifier=1) -> int:
+    """
+    :return: Unused node identifier >= start_identifier.
+    """
+    identifier = start_identifier
+    node = nodeset.findNodeByIdentifier(identifier)
+    while node.isValid():
+        identifier += 1
+        node = nodeset.findNodeByIdentifier(identifier)
+    return identifier
+
+
 def group_add_group_elements(group : FieldGroup, other_group : FieldGroup, only_dimension=None):
     '''
     Add to group elements and/or nodes from other_group.
@@ -169,6 +194,7 @@ def group_add_group_elements(group : FieldGroup, other_group : FieldGroup, only_
                 nodeset_group = node_group.getNodesetGroup()
                 nodeset_group.addNodesConditional(other_group.getFieldNodeGroup(nodeset))
 
+
 def group_get_highest_dimension(group : FieldGroup):
     '''
     Get highest dimension of elements or nodes in group.
@@ -186,6 +212,7 @@ def group_get_highest_dimension(group : FieldGroup):
         return 0
     return -1
 
+
 def identifier_ranges_fix(identifier_ranges):
     '''
     Sort from lowest to highest identifier and merge adjacent and overlapping
@@ -202,11 +229,13 @@ def identifier_ranges_fix(identifier_ranges):
         else:
             i += 1
 
+
 def identifier_ranges_from_string(identifier_ranges_string):
     '''
     Parse string containing identifiers and identifier ranges.
     Function is suitable for processing manual input with whitespace, trailing non-digits.
     Ranges are sorted so strictly increasing. Overlapping ranges are merged.
+    Future: migrate to use .. as separator for compatibility with EX file groups and cmgui.
     :param identifier_ranges_string: Identifier ranges as a string e.g. '1-30,55,66-70'.
     '30-1, 55,66-70s' also produces the same result.
     :return: Ordered list of identifier ranges e.g. [[1,30],[55,55],[66,70]]
@@ -242,6 +271,7 @@ def identifier_ranges_from_string(identifier_ranges_string):
 def identifier_ranges_to_string(identifier_ranges):
     '''
     Convert ranges to a string, contracting single object ranges.
+    Future: migrate to use .. as separator for compatibility with EX file groups and cmgui.
     :param identifier_ranges: Ordered list of identifier ranges e.g. [[1,30],[55,55],[66,70]]
     :return: Identifier ranges as a string e.g. '1-30,55,66-70'
     '''
@@ -328,6 +358,7 @@ def nodeset_group_to_identifier_ranges(nodeset_group):
     '''
     return domain_iterator_to_identifier_ranges(nodeset_group.createNodeiterator())
 
+
 def mesh_destroy_elements_and_nodes_by_identifiers(mesh, element_identifiers):
     '''
     Deletes elements and related nodes using element identifiers.
@@ -428,3 +459,79 @@ def print_node_field_parameters(value_labels, node_field_parameters, format_stri
         nodeIdentifier = nodeParameters[0]
         print('( ' + str(nodeIdentifier) + ', [ ' + ', '.join(parameter_lists_to_string(valueParameters, format_string) for valueParameters in nodeParameters[1]) + ' ] )' +  (', ' if (nodeIdentifier != lastNodeIdentifier) else ''))
     print(']\n')
+
+def disconnectFieldMeshGroupBoundaryNodes(coordinateFields, meshGroup1, meshGroup2, nextNodeIdentifier):
+    """
+    Duplicate nodes in use by coordinateField on meshGroup1 and meshGroup2 and use these exclusively in meshGroup2.
+    :param coordinateFields: The sequence of field to disconnect, type Zinc FieldFiniteElement. These are expected to
+    be define with the same Elementfieldtemplate for all components in any element.
+    :param meshGroup1: A Zinc MeshGroup containing elements in first group.
+    :param meshGroup2: A Zinc MeshGroup containing elements in second group, which will be changed to use the new copies of nodes.
+    :param nextNodeIdentifier: First available node identifier.
+    :return: Final nextNodeIdentifier to use after this call, list of created node identifiers.
+    """
+    elemiter = meshGroup1.createElementiterator()
+    element = elemiter.next()
+    nodeIdentifiers1 = set()
+    while element.isValid():
+        eft = element.getElementfieldtemplate(coordinateFields[0], -1)
+        nodeCount = eft.getNumberOfLocalNodes()
+        for n in range(1, nodeCount + 1):
+            node = element.getNode(eft, n)
+            nodeIdentifiers1.add(node.getIdentifier())
+        element = elemiter.next()
+    copyIdentifiersMap = {}
+    fieldmodule = coordinateFields[0].getFieldmodule()
+    fieldcache = fieldmodule.createFieldcache()
+    nodes = fieldmodule.findNodesetByFieldDomainType(Field.DOMAIN_TYPE_NODES)
+    nodetemplate = nodes.createNodetemplate()
+
+    allValueLabels = [
+        Node.VALUE_LABEL_VALUE, Node.VALUE_LABEL_D_DS1, Node.VALUE_LABEL_D_DS2, Node.VALUE_LABEL_D2_DS1DS2,
+        Node.VALUE_LABEL_D2_DS1DS3, Node.VALUE_LABEL_D2_DS2DS3, Node.VALUE_LABEL_D_DS3, Node.VALUE_LABEL_D3_DS1DS2DS3]
+
+    with ChangeManager(fieldmodule):
+        elemiter = meshGroup2.createElementiterator()
+        element = elemiter.next()
+        while element.isValid():
+            eft = element.getElementfieldtemplate(coordinateFields[0], -1)
+            nodeCount = eft.getNumberOfLocalNodes()
+            for n in range(1, nodeCount + 1):
+                existingNode = element.getNode(eft, n)
+                existingNodeIdentifier = existingNode.getIdentifier()
+                copyNodeIdentifier = copyIdentifiersMap.get(existingNodeIdentifier)
+                copyNode = None
+
+                if copyNodeIdentifier:
+                    copyNode = nodes.findNodeByIdentifier(copyNodeIdentifier)
+                else:
+                    if existingNodeIdentifier in nodeIdentifiers1:
+                        copyNodeIdentifier = nextNodeIdentifier
+                        for coordinateField in coordinateFields:
+                            nodetemplate.defineFieldFromNode(coordinateField, existingNode)
+                        copyNode = nodes.createNode(nextNodeIdentifier, nodetemplate)
+                        copyIdentifiersMap[existingNodeIdentifier] = copyNodeIdentifier
+                        nextNodeIdentifier += 1
+
+                        # copy field parameters from existingNode to copyNode
+                        for coordinateField in coordinateFields:
+                            componentsCount = coordinateField.getNumberOfComponents()
+
+                            for valueLabel in allValueLabels:
+                                versionCount = nodetemplate.getValueNumberOfVersions(coordinateField, -1, valueLabel)
+
+                                for version in range(1, versionCount + 1):
+                                    fieldcache.setNode(existingNode)
+                                    result, values = coordinateField.getNodeParameters(
+                                        fieldcache, -1, valueLabel, version, componentsCount)
+                                    fieldcache.setNode(copyNode)
+                                    coordinateField.setNodeParameters(
+                                        fieldcache, -1, valueLabel, version, values)
+
+                if copyNode:
+                    result = element.setNode(eft, n, copyNode)
+                    assert result == 1
+
+            element = elemiter.next()
+
+    return nextNodeIdentifier, list(copyIdentifiersMap.values())
